////////////////////////////////////////////////////////////////////// 
// merlin/MerlinSimulator.cpp 
// (c) 2000-2003 Goncalo Abecasis
// 
// This file is distributed as part of the MERLIN source code package   
// and may not be redistributed in any form, without prior written    
// permission from the author. Permission is granted for you to       
// modify this file for your own personal use, but modified versions  
// must retain this copyright notice and must not be distributed.     
// 
// Permission is granted for you to use this file to compile MERLIN.    
// 
// All computer programs have bugs. Use this file at your own risk.   
// 
// Monday March 24, 2003
// 
 
#include "MerlinSimulator.h"
#include "MapFunction.h"
#include "Random.h"

void SimulateNullData(Pedigree & ped, IntArray & markers)
   {
   IntArray alleles, vector;

   for (int f = 0; f < ped.familyCount; f++)
      {
      alleles.Dimension(ped.families[f]->count * 2);
      vector.Dimension(ped.families[f]->nonFounders * 2);

      // Create a random inheritance vector for the first marker
      for (int i = 0; i < vector.Length(); i++)
         vector[i] = globalRandom.Binary();

      double position = -1;

      for (int i = 0; i < markers.Length(); i++)
         {
         // Get information on the current marker (position, allele frequencies)
         MarkerInfo * info = ped.GetMarkerInfo(markers[i]);

         // Drop founder alleles, by sampling each chromosome
         for (int j = 0; j < ped.families[f]->founders; j++)
            for (int chr = 0; chr < 2; chr++)
               {
               int  allele = 0;
               double rand = globalRandom.Next(), sum = 0.0;

               while (sum < rand)
                  sum += info->freq[++allele];

               alleles[j * 2 + chr] = allele;
               }

#ifdef __CHROMOSOME_X__
         // All males are homozygous for X
         for (int j = 0; j < ped.families[f]->founders; j++)
            if (ped[ped.families[f]->path[j]].sex == SEX_MALE)
               alleles[j * 2 + 1] = alleles[j * 2];
#endif

         // Update the inheritance vector using the recombination fraction
         if (position != -1)
            {
            double theta = DistanceToRecombination(info->position - position);

            for (int j = 0; j < vector.Length(); j++)
               vector[j] ^= (bool) (globalRandom.Next() < theta);
            }
         position = ped.GetMarkerInfo(markers[i])->position;

         // Assign offspring genotypes using founder alleles + vector
         for (int j = 0; j < ped.families[f]->nonFounders; j++)
            {
            int index  = j + ped.families[f]->founders;
            int person = ped.families[f]->path[index];
            int father_allele = ped[person].father->traverse*2 + vector[j*2];
            int mother_allele = ped[person].mother->traverse*2 + vector[j*2+1];

            index *= 2;
            alleles[index] = alleles[mother_allele];
#ifdef __CHROMOSOME_X__
            // All males are homozygous for X
            if (ped[person].sex == SEX_MALE)
               alleles[index + 1] = alleles[index];
            else
#endif
            alleles[index + 1] = alleles[father_allele];
            }

         // Replicate the pattern of missing data in the pedigree
         for (int j = 0; j < ped.families[f]->count; j++)
            {
            int person = ped.families[f]->path[j];

            if (ped[person].markers[markers[i]].isKnown())
               {
               ped[person].markers[markers[i]][0] = alleles[j * 2];
               ped[person].markers[markers[i]][1] = alleles[j * 2 + 1];
               }
            else
               ped[person].markers[markers[i]][0] =
               ped[person].markers[markers[i]][1] = 0;
            }
         }
      }
   }

 
